#!/usr/bin/perl
#
#  Author: Walid Magdy & Preslav Nakov
#  
#  Description: Checks the file format for subtask A of SemEval-2015 Task 3.
#
#  Version: 1.0
#
#  Last modified: November 12, 2014
#
#
#  Use:
#     SemEval2015-task3-checker-Arabic.pl <XML_INPUT_FILE> <PREDICTIONS_FILE>
#
#  Example use:
#     SemEval2015-task3-checker-subtaskA.pl QA-Arabic-dev-input.xml QA-Arabic-dev-predicted-good.txt
#     SemEval2015-task3-checker-subtaskA.pl QA-Arabic-dev-input.xml QA-Arabic-dev-predicted-bad.txt
#
#  Description:
#     The format checker performs format checking for the predictions file given the XML input it is generated for.
#     Each line should contain one prediction per line in the format "<Question_ID-Comment_ID>	<Label>"
#     with a TAB as a separator, e.g.,
#
#   	52665-105341	direct
#   	52665-218107	irrelevant
#   	52665-174761	irrelevant
#   	...
#
#     The Comment IDs in the predictions file should match the Comment IDs in the XML file.
#


use warnings;
use strict;
use utf8;


################
###   MAIN   ###
################

my @XML_CIDs = ();
my $qid = 0;

### 1. Check the parameters
die "Usage: $0 <XML_INPUT_FILE> <PREDICTIONS_FILE>\n" if ($#ARGV != 1);
my $XML_FILE         = $ARGV[0];
my $PREDICTIONS_FILE = $ARGV[1];

### 2. Get the comment IDs from the training file
open XML, $XML_FILE or die "Error opening $XML_FILE!";
binmode(XML, ":utf8");
while (<XML>) {
	###		<Question QID = "20831" ...>
	if (/<Question QID \= \"(\d+)\"/){
		$qid = $1;
	}
	### 	<Answer CID = "21282" CGOLD = "?">
	elsif (/<Answer CID = \"(\d+)\"/) {
		push @XML_CIDs, "$qid-$1";
	}
}
close XML or die;


### 3. Check the file format
open PREDICTED, $PREDICTIONS_FILE or die "Error opening $PREDICTIONS_FILE!";
binmode(PREDICTED, ":utf8");
for (my $ind = 0; <PREDICTED>; $ind++) {
	
	# 3.1. Get the PREDICTED CID
	# Q2601_C1	Bad
	die "Wrong file format!" if (!/^(\d+\-\d+)\t(direct|related|irrelevant)[\n\r]*$/);
	my $CID = $1;

	# 3.2. Make sure it matches that in the XML file
	if ($XML_CIDs[$ind] ne $CID) {
		die "Wrong comment ID on line ", $ind+1, ": expected '$XML_CIDs[$ind]', found '$CID'\n";
	}
}
close PREDICTED or die;

### 4. If we are here, it is all good
print "\nThe file format looks OK.\n";
